#include "plotwindow.h"
#include <QDebug>
#include <QDateTime>

QPlotWindow::QPlotWindow(QWidget *parent, const Setup* config_, PlotData* plotdata_)
    : QMainWindow(parent),
      config(config_),
      ui(new Ui::PlotWindow),
      plotdata(plotdata_)
{
    ui->setupUi(this);
    ui->imageCanvas->setID(1);
    ui->spectrumCanvas->setID(2);
    // hide asic and channel boxes
    // for image and spectrum canvas
    ui->imageCanvas->hideBoxes();
    ui->spectrumCanvas->hideBoxes();

    ui->pairCanvas->setID(3);
    ui->pairCanvas->hideLGHGBoxes();
    ui->LGHGCanvas->setID(4);
    ui->LGHGCanvas->hidePairBox();

    connect(ui->actionStart, &QAction::triggered, this, &QPlotWindow::handleStart);
    connect(ui->actionPause, &QAction::triggered, this, &QPlotWindow::handlePause);
    connect(ui->actionStop, &QAction::triggered, this, &QPlotWindow::handleStop);
    connect(ui->actionUnzoom, &QAction::triggered, this, &QPlotWindow::handleUnzoom);
    connect(ui->actionClearAll, &QAction::triggered, this, &QPlotWindow::handleClear);

    connect(ui->imageCanvas, &QCustomCanvas::canvasZoomedout, this, &QPlotWindow::handleUnzoom);
    connect(ui->spectrumCanvas, &QCustomCanvas::canvasZoomedout, this, &QPlotWindow::handleUnzoom);
    connect(ui->imageCanvas, &QCustomCanvas::canvasCleared, this, &QPlotWindow::handleClear);
    connect(ui->spectrumCanvas, &QCustomCanvas::canvasCleared, this, &QPlotWindow::handleClear);
    connect(ui->imageCanvas, &QCustomCanvas::canvasSaved2Image, this, &QPlotWindow::handleScreenshot);
    connect(ui->spectrumCanvas, &QCustomCanvas::canvasSaved2Image, this, &QPlotWindow::handleScreenshot);
    connect(ui->imageCanvas, &QCustomCanvas::canvasSaved2Txt, this, &QPlotWindow::handleSave2txt);
    connect(ui->spectrumCanvas, &QCustomCanvas::canvasSaved2Txt, this, &QPlotWindow::handleSave2txt);

    connect(ui->pairCanvas, &QCustomCanvas::canvasZoomedout, this, &QPlotWindow::handleUnzoom);
    connect(ui->LGHGCanvas, &QCustomCanvas::canvasZoomedout, this, &QPlotWindow::handleUnzoom);
    connect(ui->pairCanvas, &QCustomCanvas::canvasCleared, this, &QPlotWindow::handleClear);
    connect(ui->LGHGCanvas, &QCustomCanvas::canvasCleared, this, &QPlotWindow::handleClear);
    connect(ui->pairCanvas, &QCustomCanvas::canvasSaved2Image, this, &QPlotWindow::handleScreenshot);
    connect(ui->LGHGCanvas, &QCustomCanvas::canvasSaved2Image, this, &QPlotWindow::handleScreenshot);
    connect(ui->pairCanvas, &QCustomCanvas::canvasSaved2Txt, this, &QPlotWindow::handleSave2txt);
    connect(ui->LGHGCanvas, &QCustomCanvas::canvasSaved2Txt, this, &QPlotWindow::handleSave2txt);

    connect(ui->LGHGCanvas, &QCustomCanvas::asicChanged, this, &QPlotWindow::handleAsicChanged);
    connect(ui->LGHGCanvas, &QCustomCanvas::channelChanged, this, &QPlotWindow::handleChannelChanged);

    connect(ui->LGHGCanvas, &QCustomCanvas::sourceChanged, this, &QPlotWindow::handleSourceChanged);
    connect(ui->pairCanvas, &QCustomCanvas::pairChanged, this, &QPlotWindow::handlePairChanged);



    // draw image
    ui->imageCanvas->Canvas()->cd();
    plotdata->mMutex.lock();
    plotdata->drawImage();
    plotdata->drawCountsLabel();
    plotdata->mMutex.unlock();
    ui->imageCanvas->Canvas()->Modified();
    ui->imageCanvas->Canvas()->Update();

    // draw spectrum
    ui->spectrumCanvas->Canvas()->cd();
    plotdata->mMutex.lock();
    plotdata->drawSpectrum();
    plotdata->mMutex.unlock();
    ui->spectrumCanvas->Canvas()->Modified();
    ui->spectrumCanvas->Canvas()->Update();

    // draw single pair spectrum
    ui->pairCanvas->Canvas()->cd();
    plotdata->mMutex.lock();
//    plotdata->drawLGspectrum(LGAsicID, LGChannelID);
    plotdata->drawPairSpectrum(pairID);
    plotdata->mMutex.unlock();
    ui->pairCanvas->Canvas()->Modified();
    ui->pairCanvas->Canvas()->Update();

    // draw LG/HG spectrum
    ui->LGHGCanvas->Canvas()->cd();
    plotdata->mMutex.lock();
    plotdata->drawLGspectrum(LGAsicID, LGChannelID);
    plotdata->mMutex.unlock();
    ui->LGHGCanvas->Canvas()->Modified();
    ui->LGHGCanvas->Canvas()->Update();
}


QPlotWindow::~QPlotWindow()
{
    delete ui;
}

void QPlotWindow::handleStart(){
    emit runStarted();
    ui->statusbar->showMessage("Running.");
}
void QPlotWindow::handlePause()
{
    emit runPaused();
    ui->statusbar->showMessage("Paused.");
}
void QPlotWindow::handleStop()
{
    emit runStopped();
    ui->statusbar->showMessage("Stopped.");
}

void QPlotWindow::handleUnzoom(int id)
{
    if (id==0)
    {
        plotdata->unZoomImage();
        plotdata->unZoomSpectrum();
        // TODO
        plotdata->unZoomPairSpectrum(pairID);
        if (source=="LG")
            plotdata->unZoomLG(LGAsicID, LGChannelID);
        else if(source == "HG")
            plotdata->unZoomHG(HGAsicID, HGChannelID);
    }
    else if (id == ui->imageCanvas->getID())
    {
        plotdata->unZoomImage();
    }
    else if (id ==ui->spectrumCanvas->getID())
    {
        plotdata->unZoomSpectrum();
    }
    else if (id ==ui->pairCanvas->getID())
    {
        plotdata->unZoomPairSpectrum(pairID);
    }
    else if (id ==ui->LGHGCanvas->getID())
    {
        if (source=="LG")
            plotdata->unZoomLG(LGAsicID, LGChannelID);
        else if(source == "HG")
            plotdata->unZoomHG(HGAsicID, HGChannelID);
    }
}

void QPlotWindow::reopen()
{
    ui->imageCanvas->Canvas()->cd();
    ui->imageCanvas->Canvas()->Modified();
    ui->imageCanvas->Canvas()->Update();

    ui->spectrumCanvas->Canvas()->cd();
    ui->spectrumCanvas->Canvas()->Modified();
    ui->spectrumCanvas->Canvas()->Update();

    ui->pairCanvas->Canvas()->cd();
    ui->pairCanvas->Canvas()->Modified();
    ui->pairCanvas->Canvas()->Update();

    ui->LGHGCanvas->Canvas()->cd();
    ui->LGHGCanvas->Canvas()->Modified();
    ui->LGHGCanvas->Canvas()->Update();
}

void QPlotWindow::changeEvent(QEvent *e)
{
    if(e->type() == QEvent::WindowStateChange){
        QWindowStateChangeEvent *event = static_cast<QWindowStateChangeEvent *>(e);
        if((event->oldState() & Qt::WindowMaximized)||
                (event->oldState() & Qt::WindowMinimized)||
                (event->oldState() & Qt::WindowNoState &&
                 this->windowState() == Qt::WindowMaximized)){
            if(ui->imageCanvas->Canvas()){
                ui->imageCanvas->Canvas()->Resize();
                ui->imageCanvas->Canvas()->Update();
            }
            if(ui->spectrumCanvas->Canvas()){
                ui->spectrumCanvas->Canvas()->Resize();
                ui->spectrumCanvas->Canvas()->Update();
            }
        }
    }
}

void QPlotWindow::redraw()
{
    plotdata->mMutex.lock();
    ui->imageCanvas->Canvas()->cd();
    std::string strtmp = "Total counts: " + std::to_string(plotdata->counts);
    plotdata->getCountsText()->SetText(0.7, 0.92, strtmp.c_str());
    ui->imageCanvas->Canvas()->Modified();
    ui->imageCanvas->Canvas()->Update();

    ui->spectrumCanvas->Canvas()->cd();
    ui->spectrumCanvas->Canvas()->Modified();
    ui->spectrumCanvas->Canvas()->Update();

    ui->pairCanvas->Canvas()->cd();
    ui->pairCanvas->Canvas()->Modified();
    ui->pairCanvas->Canvas()->Update();

    ui->LGHGCanvas->Canvas()->cd();
    ui->LGHGCanvas->Canvas()->Modified();
    ui->LGHGCanvas->Canvas()->Update();

    plotdata->mMutex.unlock();
}

void QPlotWindow::handleClear(int id){
    if (id==0){
        qDebug() << "Clear all";
        plotdata->clearAll();
    }
    else if (id ==ui->imageCanvas->getID()) {
        qDebug() << "Clear image";
        plotdata->clearImage();
    }
    else if (id ==ui->spectrumCanvas->getID()) {
        qDebug() << "Clear spectrum";
        plotdata->clearSpectrum();
    }
    else if (id == ui->pairCanvas->getID())
    {
        qDebug() << "Clear single pair spectrum";
        plotdata->clearPairSpectra();
    }
    else if (id == ui->LGHGCanvas->getID())
    {
        qDebug() << "Clear LG/HG spectrum";
        plotdata->clearLGHG();
    }
}

void QPlotWindow::handleScreenshot(int id){
    QCustomCanvas* canvas;
    QString key;
    QDateTime dateTime = QDateTime::currentDateTime();
    QString currentDateTime = dateTime.toString("yyyy-MM-dd-HH-mm-ss");
    QString fileName;
    if (id==0){
        qDebug() << "Save all";
        handleScreenshot(ui->imageCanvas->getID());
        handleScreenshot(ui->spectrumCanvas->getID());
        handleScreenshot(ui->pairCanvas->getID());
        handleScreenshot(ui->LGHGCanvas->getID());
    }
    else if (id ==ui->imageCanvas->getID()) {
        key = "Image";
        canvas = ui->imageCanvas;
        fileName = config->outputDir.absolutePath() + QString("/Screenshot-%1-%2.png").arg(key, currentDateTime);
    }
    else if (id ==ui->spectrumCanvas->getID()) {
        key = "Coincidence-Spectrum";
        canvas = ui->spectrumCanvas;
        fileName = config->outputDir.absolutePath() + QString("/Screenshot-%1-%2.png").arg(key, currentDateTime);
    }
    else if (id ==ui->pairCanvas->getID()) {
        key = "Pair-Spectrum";
        canvas = ui->pairCanvas;
        fileName = config->outputDir.absolutePath() +
                QString("/Screenshot-%1-%2-Pair-%3.png").arg(key, currentDateTime,
                                                                        QString::number(pairID));
    }
    else if (id ==ui->LGHGCanvas->getID()) {
        key = source + "-Spectrum";
        canvas = ui->LGHGCanvas;
        if(source == "LG")
            fileName = config->outputDir.absolutePath() +
                    QString("/Screenshot-%1-%2-ASIC-%3-Channel-%4.png").arg(key, currentDateTime,
                                                                            QString::number(LGAsicID), QString::number(LGChannelID));
        else if (source == "HG")
            fileName = config->outputDir.absolutePath() +
                    QString("/Screenshot-%1-%2-ASIC-%3-Channel-%4.png").arg(key, currentDateTime,
                                                                            QString::number(HGAsicID), QString::number(HGChannelID));
    }
    else
        return;

    qDebug() << QString("Save %1").arg(key);
    plotdata->mMutex.lock();
    canvas->Canvas()->SaveAs(fileName.toLocal8Bit().constData());
    plotdata->mMutex.unlock();
    ui->statusbar->showMessage(QString("%1 saved to: %2").arg(key, fileName));
}

void QPlotWindow::handleSave2txt(int id){

    QDateTime dateTime = QDateTime::currentDateTime();
    QString currentDateTime = dateTime.toString("yyyy-MM-dd-HH-mm-ss");
    if (id==0){
        qDebug() << "Save all";
    }
    else if (id == ui->imageCanvas->getID()) {
        qDebug() << "Save image to txt";
        QString fileName = config->outputDir.absolutePath() + QString("/Image-%1.txt").arg(currentDateTime);
        plotdata->saveImage2txt(fileName.toStdString());
        ui->statusbar->showMessage(QString("Image saved to: %1").arg(fileName));
    }
    else if (id == ui->spectrumCanvas->getID()) {
        qDebug() << "Save coincidence spectrum to txt";
        QString fileName = config->outputDir.absolutePath() + QString("/Coincidence-Spectrum-%1.txt").arg(currentDateTime);
        plotdata->saveSpectrum2txt(fileName.toStdString());
        ui->statusbar->showMessage(QString("Spectrum saved to: %1").arg(fileName));
    }
    else if (id == ui->pairCanvas->getID()) {
        qDebug() << "Save pai spectrum to txt";
        QString fileName = config->outputDir.absolutePath() +
                QString("/Pair-Spectrum-%1-ID-%2.txt").arg(currentDateTime,
                                                           QString::number(pairID));
        plotdata->savePairSpectrum2txt(fileName.toStdString(), pairID);
        ui->statusbar->showMessage(QString("LG Spectrum saved to: %1").arg(fileName));
    }
    else if (id == ui->LGHGCanvas->getID()) {
        qDebug() << "Save LG/HG spectrum to txt";
        QString fileName;
        if (source=="LG")
        {
            fileName = config->outputDir.absolutePath() +
                            QString("/HG-Spectrum-%1-ASIC-%2-Channel-%3.txt").arg(currentDateTime,
                                                                                  QString::number(LGAsicID), QString::number(LGChannelID));
            plotdata->saveLGSpectrum2txt(fileName.toStdString(), LGAsicID, LGChannelID);
        }
        else if (source == "HG")
        {
            fileName = config->outputDir.absolutePath() +
                            QString("/HG-Spectrum-%1-ASIC-%2-Channel-%3.txt").arg(currentDateTime,
                                                                                  QString::number(HGAsicID), QString::number(HGChannelID));
            plotdata->saveHGSpectrum2txt(fileName.toStdString(), HGAsicID, HGChannelID);
        }
        else
            return;
        ui->statusbar->showMessage(QString("LG/HG Spectrum saved to: %1").arg(fileName));
    }
}

void QPlotWindow::handlePairChanged(int id)
{
    if (id == ui->pairCanvas->getID())
    {
        int currentID = ui->pairCanvas->getPairID();
        if (currentID == pairID)
            return;
        pairID = currentID;
        switchPair(id);
    }
}

void QPlotWindow::handleAsicChanged(int id)
{
 if (id == ui->LGHGCanvas->getID())
 {
     int currentID = (source =="LG") ? LGAsicID : HGAsicID;
     if (ui->LGHGCanvas->getAsicID() == currentID)
         return;
     if (source == "LG")
         LGAsicID = ui->LGHGCanvas->getAsicID();
     else
         HGAsicID = ui->LGHGCanvas->getAsicID();
     switchLGHG(id);
 }
}

void QPlotWindow::handleChannelChanged(int id)
{
    if (id == ui->LGHGCanvas->getID())
    {
        int currentID = (source == "LG") ? LGChannelID : HGChannelID;
        if (ui->LGHGCanvas->getChannelID() == currentID)
            return;
        if (source == "LG")
            LGChannelID = ui->LGHGCanvas->getChannelID();
        else
            HGChannelID = ui->LGHGCanvas->getChannelID();
        switchLGHG(id);
    }
}

void QPlotWindow::handleSourceChanged(int id)
{
    if (id == ui->LGHGCanvas->getID())
    {
        if(ui->LGHGCanvas->getSource() == source)
            return;
        source = ui->LGHGCanvas->getSource();
        if (source == "LG")
        {
            LGAsicID = HGAsicID;
            LGChannelID = HGChannelID;
        }
        else
        {
            HGAsicID = LGAsicID;
            HGChannelID = LGChannelID;
        }
        switchLGHG(id);
//        ui->LGHGCanvas
    }
}

void QPlotWindow::switchLGHG(int id)
{
    QCustomCanvas* canvas;
    if (id == ui->LGHGCanvas->getID())
       canvas = ui->LGHGCanvas;
    else
        return;

    // redraw
    plotdata->mMutex.lock();
    canvas->Canvas()->cd();
    canvas->Canvas()->Clear(); // clear canvas
    // draw new spectrum
    if (source == "LG")
    {
        qDebug() << QString("Draw LG ASIC-%1, Channel-%2").arg(QString::number(LGAsicID), QString::number(LGChannelID));
        plotdata->drawLGspectrum(LGAsicID, LGChannelID);
    }
    else if (source == "HG")
    {

        qDebug() << QString("Draw HG ASIC-%1, Channel-%2").arg(QString::number(HGAsicID), QString::number(HGChannelID));
        plotdata->drawHGspectrum(HGAsicID, HGChannelID);
    }
    canvas->Canvas()->Modified();
    canvas->Canvas()->Update();
    plotdata->mMutex.unlock();
}

void QPlotWindow::switchPair(int id)
{
    if (id != ui->pairCanvas->getID())
        return;
    // redraw
    plotdata->mMutex.lock();
    ui->pairCanvas->Canvas()->cd();
    ui->pairCanvas->Canvas()->Clear(); // clear canvas
    // draw new spectrum
    qDebug() << QString("Draw Pair %1").arg(QString::number(pairID));
    plotdata->drawPairSpectrum(pairID);
    ui->pairCanvas->Canvas()->Modified();
    ui->pairCanvas->Canvas()->Update();
    plotdata->mMutex.unlock();
}

